/* $Id: hold.cpp 431 2011-02-14 13:24:49Z oh2gve $
 *
 * Copyright 2011 Tapio Aaltonen 
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/


// Own stuff.
#include "hold.h"
#include "packet.h"
#include "aprspacket.h"

// std stuff.
#include <list>
using std::list;



Hold::Hold(time_t const& holdtime):
holdtime_(holdtime), hold_()
{
}


Hold::~Hold()
{
	while ( !hold_.empty() )
	{
		delete hold_.front();
		hold_.pop_front();
	}
}


void Hold::put(Packet* packet)
{
	hold_.push_back(packet);
}


Packet* Hold::get()
{
	Packet* retval = NULL;
	
	if ( !hold_.empty() )
	{
		// Check timestamp of the oldest packet.
		if ( time(NULL) - hold_.front()->getAprsPacket()->getTimestamp() > holdtime_ )
		{
			// Okay, it's now old enough to be transmitted.
			retval = hold_.front();
			hold_.pop_front();
		}
	}
	
	return retval;
}


void Hold::inform(AprsPacket* packet)
{
	// Drop possible copy of this packet from our output hold.
	for ( list<Packet*>::iterator i = hold_.begin(); i != hold_.end(); ++i )
	{
		if ( *packet == *(*i)->getAprsPacket() )
		{
			delete *i;
			hold_.erase(i);
			break;
		}
	}
}
