/* $Id: kisstnc.cpp 453 2011-02-27 10:06:27Z oh2gve $
 * 
 * Copyright 2007, 2008, 2009, 2010, 2011 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

// own stuff.
#include "kisstnc.h"
#include "serial.h"
#include "cfg.h"
using Cfg::ConnectionCfg;
#include "packetlog.h"
#include "dupefilter.h"
#include "aprspacket.h"

// std stuff.
#include <string>
using std::string;
#include <vector>
using std::vector;
#include <bitset>
using std::bitset;
#include <sstream>
using std::ostringstream;

// wx stuff.
#include <wx/log.h>
#include <wx/utils.h>
#include <wx/string.h>
#include <wx/thread.h>
#include <wx/string.h>



/// KISS frame start/end marker.
char const FEND = 0xc0;


// Constructor opens serial port by calling constructor of Serial.
KissTnc::KissTnc(ConnectionCfg const& cfg, char const& conId, string const& owncall,
                 Packetlog* packetlog, DupeFilter* dupefilter, wxString const& lockDir):
Serial(cfg, conId, owncall, packetlog, dupefilter, lockDir),
device_(cfg.device), inputBuffer_(), kissFrame_(), tnc2Packet_()
{
}

KissTnc::~KissTnc()
{
}


// PUBLIC MEMBERS


// This is what KissTnc thread keeps doing.
wxThread::ExitCode KissTnc::Entry()
{
	threadStarted();
	startupDone();
	
	// We start here when main thread says ->Run() and stop the loop when it
	// says ->Delete() or an error happens.
	while ( portIsConnected() && keepGoing() )
	{
		// Check for incoming bytes.
		string tmpdata;
		if ( !readDevice(tmpdata) )
		{
			wxLogError(wxT(" serialkisstnc(") + device_ + wxT("): failed to read data, closing device."));
			break;
		}
		
		// Save and process the data.
		inputBuffer_ += tmpdata;

		// Look for packet.
		if ( inputBuffer_.size() > 0 )
		{
			// Look for complete KISS frames in our input buffer.
			string frame;
			if ( findKissFrame(frame) )
			{
				// Found a frame, get the packet from it and send to main.
				input( AprsPacket::fromKissFrame(frame) );
			}
		}
		
		// Let's see if we've got outgoing packets.
		AprsPacket const* outpacket = output();
		if ( outpacket )
		{
			if ( !writeToDevice(outpacket->getKissFrame()) )
			{
				wxLogMessage(wxT(" serialkisstnc(") + device_ + wxT("): closing device."));
				delete outpacket;
				break;
			}
			delete outpacket;
		}
		
		Sleep(10);
	}
	
	// If we're supposed to keep going, but can't, inform main.
	if ( keepGoing() )
	{
		weHasError();
	}
	
	// This thread terminates here.
	return NULL;
}



// PRIVATE


// Find KISS frame from device input buffer.
bool KissTnc::findKissFrame(string& frame)
{
	// Look for end of a frame.
	string::size_type fendPos = inputBuffer_.find(FEND);
	if ( fendPos != string::npos  )
	{
		// Check for zero-lenght frame.
		if ( fendPos == 0 )
		{
			inputBuffer_ = inputBuffer_.erase(0, 1);
			return false;
		}
		
		// Save the frame and remove it from buffer.
		frame = inputBuffer_.substr(0, fendPos);
		inputBuffer_ = inputBuffer_.erase(0, fendPos+1);
		
		return true;
	}
	
	return false;
}
