/* $Id: logserver.h 428 2011-01-26 08:03:42Z oh2gve $
 *
 * Copyright 2009, 2010, 2011 Tapio Aaltonen
 *
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/


#ifndef LOGSERVER_H
#define LOGSERVER_H


// std stuff.
#include <string>
#include <map>
#include <ctime>

// wx stuff.
#include <wx/socket.h>
#include <wx/string.h>


/// A log server.
class LogServer
{
public:
	
	/// Constructor.
	/**
	 * Params of this IGate are used in greeting message sent to every incoming
	 * connection.
	 * \param cfg Which iface to bind and which port to listen.
	 * \param ownCall Callsign of this IGate.
	 */
	LogServer(wxIPV4address const& cfg, wxString const& ownCall);

	/// Destructor.
	virtual ~LogServer();

	/// Get status of the connection.
	/**
	 * \return true The server has a fatal error.
	 * \return false Everything is ok.
	 */
	bool operator!();
	
	/// Accept new clients.
	/**
	 * Call this periodically to let new clients in.
	*/
	void accept();
	
	/// Write given line to server.
	/**
	 * Does nothing in case of existing server error.
	 */
	void write(std::string const& line);

private:

	/// Prints error description in log.
	void logError(wxSocketError const& lastError) const;
	  
	/// True if startup was successfull.
	bool initialized_;
  
	/// Callsing of the IGate.
	wxString const ownCall_;
	
	/// Local address to bind to.
	wxIPV4address masterAddr_;
	
	// Master socket.
	/**
	 * Needs to be handled dynamically because wants wxIPV4address (which only
	 * has default constructor) as a parameter to the constructor.
	 */
	wxSocketServer* masterSocket_;
	
	/// List of connected clients.
	/**
	 * Map key is string containing host:port.
	 */
	typedef std::map<wxString, wxSocketBase*> ClientList;
	
	/// Client listing.
	ClientList clients_;
	
	/// Timestamp of last heartbeat packet sent to clients.
	time_t lastHeartBeat_;
};

#endif // LOGSERVER_H
