/* $Id: icon.cpp 448 2011-02-26 07:26:08Z oh2gve $
 *
 * Copyright 2010, 2011 Tapio Aaltonen
 *
 *
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#include "icon.h"
#include "config.h"

#include <wx/taskbar.h>
#include <wx/frame.h>
#include <wx/icon.h>
#include <wx/menu.h>
#include <wx/log.h>


BEGIN_EVENT_TABLE(Icon, wxTaskBarIcon)
	EVT_MENU(ID_toggleWindow, Icon::onToggleWindow)
	EVT_MENU(ID_close, Icon::onClose)
END_EVENT_TABLE()


Icon::Icon(wxFrame* parent):
wxTaskBarIcon(), error_(false), parent_(parent)
{
	// Load icon from file.
	wxIcon myico = wxIcon(wxT("aprsg.ico"), wxBITMAP_TYPE_ICO);

	// Check how loading went.
	if ( !myico.IsOk() )
	{
		error_ = true;
	}
	else
	{
		// It went okay, let's deploy the icon.
		if ( !SetIcon(myico, wxT(PACKAGE)) )
		{
			error_ = true;
		}
	}
}

Icon::~Icon()
{
	RemoveIcon();
}

wxMenu* Icon::CreatePopupMenu()
{
	// Generate empty menu.
	wxMenu* retval = new wxMenu();
	if ( !retval )
	{
		wxSafeShowMessage(wxT("Error"), wxT("Failed generate popup menu for taskbar icon."));
		return NULL;
	}

	// Add items in it.
//	if ( parent_->IsShown() )
//	{
//		retval->Append(ID_toggleWindow, wxT("Hide"));
//	}
//	else
//	{
//		retval->Append(ID_toggleWindow, wxT("Show"));
//	}
	retval->Append(ID_close, wxT("Close"));

	return retval;
}

bool Icon::operator!() const
{
	return error_;
}

void Icon::onToggleWindow(wxCommandEvent& event)
{
	if ( parent_->IsShown() )
	{
		parent_->Hide();
	}
	else
	{
		parent_->Show();
	}
}

void Icon::onClose(wxCommandEvent& event)
{
	parent_->Close();
}
