/* $Id: main.cpp 450 2011-02-26 09:12:44Z oh2gve $
 *
 * Copyright 2010, 2011 Tapio Aaltonen
 *
 *
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

// Own stuff.
#include "../main.h"
#include "window.h"
#include "controller.h"
#include "config.h"
#include "../cfg.h"
#include <fap.h>

// Wx stuff.
#include <wx/app.h>
#include <wx/socket.h>
#include <wx/log.h>


AprsApp::AprsApp():
wxApp(), window_(NULL), controller_(NULL)
{
}

AprsApp::~AprsApp()
{
}

bool AprsApp::OnInit()
{
	// Just for the lulz.
	if ( !wxApp::OnInit() )
	{
		return false;
	}
	
	// This hack must be called from main thread in order to use wxSocketBase
	// in secondary threads under MSW.
	wxSocketBase::Initialize();

	// Create the main window.
	window_ = new Window( wxString(wxT(PACKAGE)) + wxString(wxT(" ")) + wxString(wxT(PACKAGE_VERSION)) );
	if ( window_ == NULL )
	{
		wxLogError(wxT("Failed to create status window."));
		return false;
	}

	// Show main window.
	//window_->Show();
	SetTopWindow(window_);

	// Initialize The Parser. It is needed to validate some config values.
	fap_init();

	// Read configuration from file.
	if ( !Cfg::readCfg(wxT("aprsg.ini"), configuration_) )
	{
		fap_cleanup();
		return false;
	}

	// Open main log file.
	if ( !openLog(true) )
	{
		fap_cleanup();
		return false;
	}

	// Start assistant threads.
	if ( !startAssistants() )
	{
		closeLog();
		fap_cleanup();
		return false;
	}

	// Open uplink and local connections.
	if ( !openConnections() )
	{
		closeConnections();
		stopAssistants();
		closeLog();
		fap_cleanup();
		return false;
	}

	wxLogMessage(wxT("All connections running, starting controller."));

	// Create controller thread.
	controller_ = new Controller(window_, configuration_, uplink_, connections_, packetlog_);
	if ( controller_->Create() != wxTHREAD_NO_ERROR )
	{
		wxLogError(wxT("Failed to create controller."));
		delete controller_;
		return false;
	}

	// Start controller thread.
	if ( controller_->Run() != wxTHREAD_NO_ERROR )
	{
		wxLogError(wxT("Failed to start controller."));
		delete controller_;
		return false;
	}

	// Event loop starts after this method returns.
	return true;
}


int AprsApp::OnExit()
{
	// Terminate controller.
	if ( controller_ != NULL )
	{
		controller_->Delete();
		delete controller_;
	}

	// Terminate connections.
	closeConnections();

	// Other stuff.
	closeLog();
	fap_cleanup();

	return wxApp::OnExit();
}

IMPLEMENT_APP(AprsApp)
